import pygame
import random
import math

# Provides a sprite that can be moved around the screen
# The sprite can be told to move in a particular direction
# and will move in that direction each time update is called
# The sprite will not leave the screen area

class Cracker:

    # Initialise the Cracker object
    #    position - the initial position of the player
    #    limit - a tuple giving the size of the screen
    #    speed - a tuple giving the speed of movement in pixels/second
    #    image - the image to be drawn
    #    rotation - the rotation amount
    #    width - the x size of the image in pixels
    #    target - the object the cracker is chasing
    def __init__(self, position, limit, speed, rotation, image, width, target):

        # Set the target for the chase
        self.target = target

        self.rotateAngle = 0
        self.rotation = rotation
        
        # record the initial position as the reset position
        self.resetPosition = position

        # Scale and store the image in a property called image
        aspect = image.get_width()/image.get_height()
        height = width * aspect
        self.image = pygame.transform.scale(image,(int(width),int(height)))
        self.original = self.image
        
        # Create a rectangle which is the size of the player
        # We use the dimensions of the image to work out the player size
        self.rect = pygame.Rect(0,0,self.image.get_width(), self.image.get_height())

        # Copy the screen size limits into a local variable
        self.limit = [float(limit[0]), float(limit[1])]

        # Set the movement speed of the player to 200 pixels per second
        # In a later version we might have different player speeds
        self.movementSpeed = [float(speed[0]), float(speed[1])]

        # Set the target for the chase
        self.target = target
        
        # Reset the player
        self.reset()
                    
    # Draws the player on the screen at its current position
    def draw(self, surface):
        surface.blit(self.image, self.rect)

    # Updates the state of the player
    #     deltaTime - the time in seconds since update was last called
    #                 usually (but not always) a 120th of a second
    def update(self, deltaTime):

        oldCenter = self.rect.center
        
        self.rotateAngle = self.rotateAngle+self.rotation*deltaTime
        self.image = pygame.transform.rotate(self.original,self.rotateAngle)
        self.rect = self.image.get_rect()
        self.center = oldCenter

        # Chase the target
        
        cpos = self.getCentre()
        tpos = self.target.getCentre()
        if(cpos[0]<tpos[0]):
            # sprite is to the left of the target
            # must move right towards it
            self.StopMoveLeft()
            self.StartMoveRight()
        else:
            # Go the other way
            self.StopMoveRight()
            self.StartMoveLeft()

        if(cpos[1]>tpos[1]):
            # sprite is below the target
            # must move up towards it
            self.StopMoveDown()
            self.StartMoveUp()
        else:
            # Go the other way
            self.StopMoveUp()
            self.StartMoveDown()
        
        # Handle movement - if we are moving in that direction
        # change the current position by the speed value
        
        if self.movingUp:
            self.position[1] = self.position[1] - (self.movementSpeed[1]*deltaTime)
        if self.movingDown:
            self.position[1] = self.position[1] + (self.movementSpeed[1]*deltaTime)
        if self.movingLeft:
            self.position[0] = self.position[0] - (self.movementSpeed[0]*deltaTime)
        if self.movingRight:
            self.position[0] = self.position[0] + (self.movementSpeed[0]*deltaTime)

        # Clamp the position values so we can't move off the screen

        if self.position[0] < 0:
            self.position[0]=0
        if self.position[1] < 0:
            self.position[1]=0
        if self.position[0] + self.image.get_width() > self.limit[0]:
            self.position[0] = self.limit[0] - self.image.get_width()
        if self.position[1] + self.image.get_height() > self.limit[1]:
            self.position[1] = self.limit[1] - self.image.get_height()

        # Move to the new position

        self.setPosition(self.position)

        # see if we have hit the cheese
        
        if self.rect.colliderect(self.target.rect):
            self.target.cheeseDead = True

    # Movement controllers - two for each direction
    
    def StartMoveUp(self):
        self.movingUp = True
    def StopMoveUp(self):
        self.movingUp = False
        
    def StartMoveDown(self):
        self.movingDown = True
    def StopMoveDown(self):
        self.movingDown = False

    def StartMoveLeft(self):
        self.movingLeft = True
    def StopMoveLeft(self):
        self.movingLeft = False

    def StartMoveRight(self):
        self.movingRight = True
    def StopMoveRight(self):
        self.movingRight = False

    # Sets the position of the sprite on the screen
    # Also updates the rectangle that defines the screen area
    # occupied by the sprite
    def setPosition(self,position):
        self.position = [float(position[0]), float(position[1])]
        self.rect.center = self.position

    # Sets the movement speed of the sprite
    #     speed - tuple containing positive x and y speed values
    #             
    def setSpeed(self,speed):
        self.movementSpeed = [float(speed[0]), float(speed[1])]

    # Resets the state of the sprite for a new game
    # Puts it back to start position and clears all the
    # movement flags
    def reset(self):
        self.setPosition(self.resetPosition)
        self.movingUp = False
        self.movingDown = False
        self.movingLeft = False
        self.movingRight = False
               
    # Gets the name of this sprite
    def getName(self):
        return "cracker"

    # Gets the description of this sprite
    def getDescription(self):
        return "A moving sprite that will not leave the playfield"

    # Gets the centre of this sprite as a tuple
    def getCentre(self):
        x = self.position[0] + (self.image.get_width()/2)
        y = self.position[1] + (self.image.get_height()/2)
        return (x,y)

    # Gets the distance from a sprite
    def getDistanceFrom(self, s):
        p1 = self.getCentre()
        p2 = s.getCentre()
        dx = p1[0] - p2[0]
        dy = p1[1] - p2[1]
        return math.sqrt((dx*dx) + (dy*dy))
    
    def setTarget(self,newTarget):
        self.target = newTarget

    
